<?
  /*
     The function get_materialtraits returns an associative array
     containing all information contained in the material database.
     The returned array contains a schema designed to minimize redundant
     information as well as playing nicely with future information expansion.

     The schema is designed as a potentially recursive lookup based on the
     contents of the key/value pairs. Recursion only occurs at the leaf nodes
     of the tree and only if the value of the leaf node is present as an
     immediate sucessor of the root node. This allows arbitrary information
     association with possibly unrelated (schema) values.

     The 'material' key of the materailtraits array is the root of the
     information tree and is refererred to as 'root' for the duration of this
     document. In the examples, text enclosed in '[]' represents the key of
     an element in the array and the value follows '=>'. The word 'Array'
     represents another associative array.

     A nonsensical example is given for exhibition purposes.

     <eg>
       Array
       (
         [material] => Array
           (
             [steel] => Array
               (
                 [density] => steel
                 [name] => steel
                 [hardness] => steel
               )
             [steeluminum] = Array
               (
                 [density] => aluminum
                 [name] => steeluminum
                 [hardness] => steel
               )
           )
         [density] => Array
           (
             [steel] => Array
               (
                 [value] => 234
                 [system] => SI
               )
             [steeluminum] = Array
               (
                 [value] => 123
                 [system] => SI
               )
           )
         ...
       )


     Information is obtained from the tree as the root ('material' element)
     is traversed to a particular leaf, recursively continuing iff both
     the key of the leaf element exists as an immediate sucessor of the
     root element AND the leaf value exists as a child of the former.
     (If only one of the above cases are true, then the schema is corrupt.)

     This seems complicated but it really is not and eliminates a large
     amount of redundant information in the tree. An explanation of the
     above example will help.

     We wish to find out all the information contained in the database DB.
     DB["material"] is the root and the value of DB["material"] contains an
     associative array containing the materials. Let's start with "steel" in
     the above example ( Array["material"] => Array["steel"] ). The "steel"
     element is not empty so we continue the traversal to "density"
     (Array["material"] => Array["steel"] => Array["density"] => steel).
     
     This element is empty, so we check to see if we must recurse to find
     out additional information. Before we do, lets recap to find out what
     we've learned so far: There is an element in the material database
     with the tag "steel" and there is some information (we are not sure
     how much yet) associated with tag "density". So to determine if there is
     additional information associated with the key 'density' and the value
     'steel', we look for the key 'density' with the value 'steel' as an
     immediate sucessor of the root. (ie does DB['density'] => Array['steel']
     exist? ) We see that it does, so we continue the traversal and come to
     two additional sub-values: 'value' and 'system' neither which exist as
     immediate sucessors of the root. These values are therefore terminals.

     Let's recap what we know so far: We started with the tag 'steel' and have
     found metadata 'density', 'steel', and actual values 'value' 234 and
     'system' SI. The second 'steel' seems redundant but it's usefullness
     will come to light in a moment.

     There is additional information associated at the same level of the
     'density' key under steel ( Array["material"] => Array["steel"] ),
     namely 'name' and 'hardness' both without immediate successors under the
     root node with that name so they are also terminals (although one could
     envision further expansion like 'density').

     We have completed the traversal of 'steel' under materials so let's go
     over what we know: We have the material tag 'steel' with the sub-tag
     'density' that is described by tags 'steel', and further described by
     a value '234' and a system 'SI'. Additional information for 'steel' is
     its name 'steel' and hardness 'steel' (which should have futher info
     attached in the real world)

     We now continue the traversal under the root node to a (fictional)
     material 'steeluminum'. If we follow the above traversal example, we
     notice that it behaves the same as steel with the exception that the
     'hardness' element is mapped to 'steel' instead of 'steeluminum'.
     (density of aluminum with the hardness of steel). This show the 
     power and purpose of this system. We can make arbitrary associations
     between data in a compact form.
*/

  error_reporting(E_ALL);

  //connect to the database
  function db_connect($db)
  {
    $connection = @mysql_connect("localhost","newton","apple");

    if(!$connection || !mysql_select_db($db,$connection)) {
      session_write_close();
      header("Location: http://mater.brlcad.org/error/db_connect_error.html");
      exit;
    }

    return $connection;
  }

  function query_redirect($query,$connection)
  {
//print("\n$query\n");
    if(!($result = mysql_query($query,$connection))) {
      session_write_close();
      print("<br>$query\n".mysql_error($connection));
      //header("Location: http://mater.brlcad.org/error/server_error.html?common_inc=134");
      exit;       
    }
    return $result;
  }

  /*
     Get the database content that we can see, nothing else
  */
  function get_db($connection)
  {
    $traits = array();

/*
    $query = "select table_name from trait_tables";

    $result = query_redirect($query,$connection);

    while($row = mysql_fetch_array($result,MYSQL_ASSOC)) {



      $query = "select * from `${row["table_name"]}` where id in
        (select materialID from traits where 
        table_name = '${row["table_name"]}' and (trait_private = '0' or
        userID = '${_SESSION["uid"]}'))";

      $result2 = query_redirect($query,$connection);
      $trait_tbl = array();
      while($row2 = mysql_fetch_array($result2,MYSQL_ASSOC)) {
        $id = $row2["id"];
        unset($row2["id"]);
        $trait_tbl[$id] = $row2;
      }
*/
      $query = "select traits.table_name, materialID from materials
        join traits on
          traits.materialID=materials.id
           and (traits.trait_private = '0' or
             traits.userID = '${_SESSION["uid"]}')
         where (materials.mat_private = '0' or
           materials.userID = '${_SESSION["uid"]}')
       order by table_name, materialID";

      $result = query_redirect($query,$connection);
      while($row = mysql_fetch_array($result,MYSQL_ASSOC)) {
        if (!isset($traits[$row["table_name"]])) {
          $traits[$row["table_name"]] = array();
        }
        $trait_tbl = $traits[$row["table_name"]];
        $query = "select * from `${row["table_name"]}` where id = ${row["materialID"]}";
        $result2 = query_redirect($query,$connection);
        while($row2 = mysql_fetch_array($result2,MYSQL_ASSOC)) {
          $id = $row2["id"];
          unset($row2["id"]);
          $trait_tbl[$id] = $row2;
        }
        

      $traits[$row["table_name"]] = $trait_tbl;
    }
    

    


    $query = "select * from materials where
      userID = '${_SESSION["uid"]}' or mat_private = '0'";

    $result = query_redirect($query,$connection);

    $materials = array();
    while($row = mysql_fetch_array($result,MYSQL_ASSOC)) {
      $id = $row["id"];
      unset($row["id"]);
      $materials[$id] = $row;
    }

    $query = "select * from userinfo where uid in
      (select userID from materials where userID = '${_SESSION["uid"]}' or
      mat_private = '0' union select userID from traits where
      userID = '${_SESSION["uid"]}' or trait_private = '0' union
      select '${_SESSION["uid"]}')";

    $result = query_redirect($query,$connection);

    $userinfo = array();
    while($row = mysql_fetch_array($result,MYSQL_ASSOC)) {
      $uid = $row["uid"];
      unset($row["uid"]);
      $userinfo[$uid] = $row;
    }

    $query = "select * from traits where (trait_private = '0' or userID =
      '${_SESSION["uid"]}') and materialID in (select id from materials where
      mat_private = '0' or userID = '${_SESSION["uid"]}')";
  
    $result = query_redirect($query,$connection);

    $traittables = array();

    //put each trait into its own array
    $traitarr = array();
    while($row = mysql_fetch_array($result,MYSQL_ASSOC)) {
      if(!isset($traitarr[$row["table_name"]]))
        $traitarr[$row["table_name"]] = array();

      $traitarr[$row["table_name"]][$row["materialID"]] = $row;
    }

//print("!!!!!!!!!!!!!!!!!!!!!!!!!!");
//print_r($traitarr);
//print("!!!!!!!!!!!!!!!!!!!!!!!!!!");
//print_r($traits);
//print("!!!!!!!!!!!!!!!!!!!!!!!!!!");

    //!!!WARNING NO INFINITE LOOP PROTECTION!!!
    foreach($traitarr as $table_name => $tablearr) {
      foreach($tablearr as $materialID => $traitval) {
        if(!isset($materials[$materialID]["traits"]))
          $materials[$materialID]["traits"] = array();
      
        //follow symbolic links if necessary...
        //the "actual" value is when the table_materialID = materialID
        $table_materialID = $traitval["table_materialID"];
        $sym_row = $traitval;
        while($sym_row["materialID"] != $table_materialID) {
          $sym_row = $tablearr[$table_materialID];
          $table_materialID = $tablearr[$table_materialID]["table_materialID"];
        }

        $materials[$materialID]["traits"][$table_name] =
          $traits[$table_name][$sym_row["materialID"]]; 

        $materials[$materialID]["traits"][$table_name]["userID"] =
          $traitval["userID"];

        $materials[$materialID]["traits"][$table_name]["trait_private"] =
          $traitval["trait_private"];

        $materials[$materialID]["traits"][$table_name]["materialID"] =
          $traitval["table_materialID"];
          
      }  
    }

    $db["materials"] = $materials;
    $db["userinfo"] = $userinfo;
    $db["traits"] = $traits;

    return $db;
  }

  /*
    Like mysql_num_rows
    Asserts that the number of rows in $result match some predicate
    $n >= 0
      $result has exactly $n rows
    $n < 0
      $n has at least abs(n) rows
  */
  function assert_num_rows($result,$n)
  {
    $rows = mysql_num_rows($result);
    $valid = (($n < 0)?(rows >= abs($n)):($n == $rows));
    if(!$valid) {
      session_write_close();
print("assert_num_rows failure, got $rows, expected $n\n");
print_r(debug_print_backtrace());
      header("Location: http://mater.brlcad.org/error/server_error.html?common_inc=269");
      exit;
    }
  }

  /*
    Like mysql_affected_rows
    Asserts that the last insert, update, or delete on $connection matched
    some predicate
    $n >= 0
      $result has exactly $n rows
    $n < 0
      $n has at least abs(n) rows
  */
  function assert_affected_rows($connection,$n)
  {
    $rows = mysql_affected_rows($connection);
    $valid = (($n < 0)?(rows >= abs($n)):($n == $rows));
    if(!$valid) {
print("assert_affected_rows failure, got $rows, expected $n\n");
print_r(debug_print_backtrace());
      session_write_close();
      header("Location: http://mater.brlcad.org/error/server_error.html?admin_inc=40");
      exit;
    }
  }

?>
